@@ -32,7 +32,10 @@
 #include "flow.h"
 
 #include "util-debug.h"
+#include "util-print.h"
+#include "util-profiling.h"
 
+//#define PRINT
 extern uint8_t engine_mode;
 
 /** \brief Get the active app layer proto from the packet
@@ -102,6 +105,198 @@ void *AppLayerGetProtoStateFromFlow(Flow *f) {
 /** global app layer detection context */
 extern AlpProtoDetectCtx alp_proto_ctx;
 
+/**
+ *  \brief Handle a chunk of TCP data
+ *
+ *  If the protocol is yet unknown, the proto detection code is run first.
+ *
+ *  \param dp_ctx Thread app layer detect context
+ *  \param f Flow
+ *  \param ssn TCP Session
+ *  \param data ptr to reassembled data
+ *  \param data_len length of the data chunk
+ *  \param flags control flags
+ *
+ *  \retval 0 ok
+ *  \retval -1 error
+ */
+int AppLayerHandleTCPData(AlpProtoDetectThreadCtx *dp_ctx, Flow *f,
+        TcpSession *ssn, uint8_t *data, uint32_t data_len, uint8_t flags)
+{
+    SCEnter();
+
+    uint16_t alproto = ALPROTO_UNKNOWN;
+    int r = 0;
+
+    BUG_ON(f == NULL);
+    BUG_ON(ssn == NULL);
+
+    alproto = f->alproto;
+
+    SCLogDebug("data_len %u flags %02X", data_len, flags);
+    if (!(f->flags & FLOW_NO_APPLAYER_INSPECTION)) {
+        /* if we don't know the proto yet and we have received a stream
+         * initializer message, we run proto detection.
+         * We receive 2 stream init msgs (one for each direction) but we
+         * only run the proto detection once. */
+        if (alproto == ALPROTO_UNKNOWN && flags & STREAM_GAP) {
+            ssn->flags |= STREAMTCP_FLAG_APPPROTO_DETECTION_COMPLETED;
+            SCLogDebug("ALPROTO_UNKNOWN flow %p, due to GAP in stream start", f);
+            StreamTcpSetSessionNoReassemblyFlag(ssn, 0);
+        } else if (alproto == ALPROTO_UNKNOWN && flags & STREAM_START) {
+            SCLogDebug("Stream initializer (len %" PRIu32 ")", data_len);
+#ifdef PRINT
+            if (data_len > 0) {
+                printf("=> Init Stream Data (app layer) -- start %s%s\n",
+                        flags & STREAM_TOCLIENT ? "toclient" : "",
+                        flags & STREAM_TOSERVER ? "toserver" : "");
+                PrintRawDataFp(stdout, data, data_len);
+                printf("=> Init Stream Data -- end\n");
+            }
+#endif
+
+            PACKET_PROFILING_APP_PD_START(dp_ctx);
+            alproto = AppLayerDetectGetProto(&alp_proto_ctx, dp_ctx, f,
+                    data, data_len, flags, IPPROTO_TCP);
+            PACKET_PROFILING_APP_PD_END(dp_ctx);
+
+            if (alproto != ALPROTO_UNKNOWN) {
+                /* store the proto and setup the L7 data array */
+                FlowL7DataPtrInit(f);
+                f->alproto = alproto;
+                ssn->flags |= STREAMTCP_FLAG_APPPROTO_DETECTION_COMPLETED;
+
+                PACKET_PROFILING_APP_START(dp_ctx, alproto);
+                r = AppLayerParse(f, alproto, flags, data, data_len);
+                PACKET_PROFILING_APP_END(dp_ctx, alproto);
+            } else {
+                if (flags & STREAM_TOSERVER) {
+                    SCLogDebug("alp_proto_ctx.toserver.max_len %u", alp_proto_ctx.toserver.max_len);
+                    if (f->flags & FLOW_TS_PM_PP_ALPROTO_DETECT_DONE) {
+                        ssn->flags |= STREAMTCP_FLAG_APPPROTO_DETECTION_COMPLETED;
+                        SCLogDebug("ALPROTO_UNKNOWN flow %p", f);
+                        StreamTcpSetSessionNoReassemblyFlag(ssn, 0);
+                    }
+                } else {
+                    if (f->flags & FLOW_TC_PM_PP_ALPROTO_DETECT_DONE) {
+                        ssn->flags |= STREAMTCP_FLAG_APPPROTO_DETECTION_COMPLETED;
+                        SCLogDebug("ALPROTO_UNKNOWN flow %p", f);
+                        StreamTcpSetSessionNoReassemblyFlag(ssn, 1);
+                    }
+                }
+            }
+        } else {
+            SCLogDebug("stream data (len %" PRIu32 " alproto "
+                    "%"PRIu16" (flow %p)", data_len, alproto, f);
+#ifdef PRINT
+            if (data_len > 0) {
+                printf("=> Stream Data (app layer) -- start %s%s\n",
+                        flags & STREAM_TOCLIENT ? "toclient" : "",
+                        flags & STREAM_TOSERVER ? "toserver" : "");
+                PrintRawDataFp(stdout, data, data_len);
+                printf("=> Stream Data -- end\n");
+            }
+#endif
+            /* if we don't have a data object here we are not getting it
+             * a start msg should have gotten us one */
+            if (alproto != ALPROTO_UNKNOWN) {
+                PACKET_PROFILING_APP_START(dp_ctx, alproto);
+                r = AppLayerParse(f, alproto, flags, data, data_len);
+                PACKET_PROFILING_APP_END(dp_ctx, alproto);
+            } else {
+                SCLogDebug(" smsg not start, but no l7 data? Weird");
+            }
+        }
+    } else {
+        SCLogDebug("FLOW_AL_NO_APPLAYER_INSPECTION is set");
+    }
+
+    SCReturnInt(r);
+}
+
+/**
+ *  \brief Attach a stream message to the TCP session for inspection
+ *         in the detection engine.
+ *
+ *  \param dp_ctx Thread app layer detect context
+ *  \param smsg Stream message
+ *
+ *  \retval 0 ok
+ *  \retval -1 error
+ */
+int AppLayerHandleTCPMsg(AlpProtoDetectThreadCtx *dp_ctx, StreamMsg *smsg)
+{
+    SCEnter();
+
+#ifdef PRINT
+    printf("=> Stream Data (raw reassembly) -- start %s%s\n",
+            smsg->flags & STREAM_TOCLIENT ? "toclient" : "",
+            smsg->flags & STREAM_TOSERVER ? "toserver" : "");
+    PrintRawDataFp(stdout, smsg->data.data, smsg->data.data_len);
+    printf("=> Stream Data -- end\n");
+#endif
+    SCLogDebug("smsg %p", smsg);
+    BUG_ON(smsg->flow == NULL);
+
+    TcpSession *ssn = smsg->flow->protoctx;
+    if (ssn != NULL) {
+        SCLogDebug("storing smsg %p in the tcp session", smsg);
+
+        /* store the smsg in the tcp stream */
+        if (smsg->flags & STREAM_TOSERVER) {
+            SCLogDebug("storing smsg in the to_server");
+
+            /* put the smsg in the stream list */
+            if (ssn->toserver_smsg_head == NULL) {
+                ssn->toserver_smsg_head = smsg;
+                ssn->toserver_smsg_tail = smsg;
+                smsg->next = NULL;
+                smsg->prev = NULL;
+            } else {
+                StreamMsg *cur = ssn->toserver_smsg_tail;
+                cur->next = smsg;
+                smsg->prev = cur;
+                smsg->next = NULL;
+                ssn->toserver_smsg_tail = smsg;
+            }
+        } else {
+            SCLogDebug("storing smsg in the to_client");
+
+            /* put the smsg in the stream list */
+            if (ssn->toclient_smsg_head == NULL) {
+                ssn->toclient_smsg_head = smsg;
+                ssn->toclient_smsg_tail = smsg;
+                smsg->next = NULL;
+                smsg->prev = NULL;
+            } else {
+                StreamMsg *cur = ssn->toclient_smsg_tail;
+                cur->next = smsg;
+                smsg->prev = cur;
+                smsg->next = NULL;
+                ssn->toclient_smsg_tail = smsg;
+            }
+        }
+
+        /* flow is free again */
+        FlowDecrUsecnt(smsg->flow);
+        /* dereference the flow */
+        smsg->flow = NULL;
+    } else { /* no ssn ptr */
+        /* if there is no ssn ptr we won't
+         * be inspecting this msg in detect
+         * so return it to the pool. */
+
+        /* flow is free again */
+        FlowDecrUsecnt(smsg->flow);
+
+        /* return the used message to the queue */
+        StreamMsgReturnToPool(smsg);
+    }
+
+    SCReturnInt(0);
+}
+
+#if 0
 /**
  *  \brief Handle a app layer TCP message
  *
@@ -128,17 +323,7 @@ int AppLayerHandleMsg(AlpProtoDetectThreadCtx *dp_ctx, StreamMsg *smsg)
     if (ssn != NULL) {
         alproto = smsg->flow->alproto;
 
-        /* Copy some needed flags */
-        if (smsg->flags & STREAM_TOSERVER)
-            smsg->flow->alflags |= FLOW_AL_STREAM_TOSERVER;
-        if (smsg->flags & STREAM_TOCLIENT)
-            smsg->flow->alflags |= FLOW_AL_STREAM_TOCLIENT;
-        if (smsg->flags & STREAM_GAP)
-            smsg->flow->alflags |= FLOW_AL_STREAM_GAP;
-        if (smsg->flags & STREAM_EOF)
-            smsg->flow->alflags |= FLOW_AL_STREAM_EOF;
-
-        if (!(smsg->flow->alflags & FLOW_AL_NO_APPLAYER_INSPECTION)) {
+        if (!(smsg->flow->flags & FLOW_NO_APPLAYER_INSPECTION)) {
             /* if we don't know the proto yet and we have received a stream
              * initializer message, we run proto detection.
              * We receive 2 stream init msgs (one for each direction) but we
@@ -151,31 +336,30 @@ int AppLayerHandleMsg(AlpProtoDetectThreadCtx *dp_ctx, StreamMsg *smsg)
                 //PrintRawDataFp(stdout, smsg->init.data, smsg->init.data_len);
                 //printf("=> Init Stream Data -- end\n");
 
-                alproto = AppLayerDetectGetProto(&alp_proto_ctx, dp_ctx,
+                alproto = AppLayerDetectGetProto(&alp_proto_ctx, dp_ctx, f
                         smsg->data.data, smsg->data.data_len, smsg->flow->alflags, IPPROTO_TCP);
                 if (alproto != ALPROTO_UNKNOWN) {
                     /* store the proto and setup the L7 data array */
                     FlowL7DataPtrInit(smsg->flow);
                     smsg->flow->alproto = alproto;
                     ssn->flags |= STREAMTCP_FLAG_APPPROTO_DETECTION_COMPLETED;
-                    smsg->flow->alflags |= FLOW_AL_PROTO_DETECT_DONE;
 
                     r = AppLayerParse(smsg->flow, alproto, smsg->flow->alflags,
                             smsg->data.data, smsg->data.data_len);
                 } else {
                     if (smsg->flags & STREAM_TOSERVER) {
                         if (smsg->data.data_len >= alp_proto_ctx.toserver.max_len) {
+                            /* protocol detection has failed */
                             ssn->flags |= STREAMTCP_FLAG_APPPROTO_DETECTION_COMPLETED;
-                            smsg->flow->alflags |= FLOW_AL_PROTO_DETECT_DONE;
+                            smsg->flow->alflags |= FLOW_AL_NO_APPLAYER_INSPECTION;
                             SCLogDebug("ALPROTO_UNKNOWN flow %p", smsg->flow);
-                            StreamTcpSetSessionNoReassemblyFlag(ssn, 0);
                         }
                     } else if (smsg->flags & STREAM_TOCLIENT) {
                         if (smsg->data.data_len >= alp_proto_ctx.toclient.max_len) {
+                            /* protocol detection has failed */
                             ssn->flags |= STREAMTCP_FLAG_APPPROTO_DETECTION_COMPLETED;
-                            smsg->flow->alflags |= FLOW_AL_PROTO_DETECT_DONE;
+                            smsg->flow->alflags |= FLOW_AL_NO_APPLAYER_INSPECTION;
                             SCLogDebug("ALPROTO_UNKNOWN flow %p", smsg->flow);
-                            StreamTcpSetSessionNoReassemblyFlag(ssn, 1);
                         }
                     }
                 }
@@ -204,7 +388,9 @@ int AppLayerHandleMsg(AlpProtoDetectThreadCtx *dp_ctx, StreamMsg *smsg)
         /* store the smsg in the tcp stream */
         if (smsg->flags & STREAM_TOSERVER) {
             SCLogDebug("storing smsg in the to_server");
-
+#if 0
+            PrintRawDataFp(stdout,smsg->data.data,smsg->data.data_len);
+#endif
             /* put the smsg in the stream list */
             if (ssn->toserver_smsg_head == NULL) {
                 ssn->toserver_smsg_head = smsg;
@@ -256,6 +442,7 @@ int AppLayerHandleMsg(AlpProtoDetectThreadCtx *dp_ctx, StreamMsg *smsg)
 
     SCReturnInt(r);
 }
+#endif
 
 /**
  *  \brief Handle a app layer UDP message
@@ -284,17 +471,18 @@ int AppLayerHandleUdp(AlpProtoDetectThreadCtx *dp_ctx, Flow *f, Packet *p)
 
     alproto = f->alproto;
 
+    uint8_t flags = 0;
     if (p->flowflags & FLOW_PKT_TOSERVER) {
-        f->alflags |= FLOW_AL_STREAM_TOSERVER;
+        flags |= STREAM_TOSERVER;
     } else {
-        f->alflags |= FLOW_AL_STREAM_TOCLIENT;
+        flags |= STREAM_TOCLIENT;
     }
 
     /* if we don't know the proto yet and we have received a stream
      * initializer message, we run proto detection.
      * We receive 2 stream init msgs (one for each direction) but we
      * only run the proto detection once. */
-    if (alproto == ALPROTO_UNKNOWN && !(f->alflags & FLOW_AL_PROTO_DETECT_DONE)) {
+    if (alproto == ALPROTO_UNKNOWN && !(f->flags & FLOW_ALPROTO_DETECT_DONE)) {
         SCLogDebug("Detecting AL proto on udp mesg (len %" PRIu32 ")",
                     p->payload_len);
 
@@ -302,19 +490,18 @@ int AppLayerHandleUdp(AlpProtoDetectThreadCtx *dp_ctx, Flow *f, Packet *p)
         //PrintRawDataFp(stdout, smsg->init.data, smsg->init.data_len);
         //printf("=> Init Stream Data -- end\n");
 
-        alproto = AppLayerDetectGetProto(&alp_proto_ctx, dp_ctx,
-                        p->payload, p->payload_len, f->alflags, IPPROTO_UDP);
+        alproto = AppLayerDetectGetProto(&alp_proto_ctx, dp_ctx, f,
+                        p->payload, p->payload_len, flags, IPPROTO_UDP);
         if (alproto != ALPROTO_UNKNOWN) {
             /* store the proto and setup the L7 data array */
             FlowL7DataPtrInit(f);
             f->alproto = alproto;
-            f->alflags &= ~FLOW_AL_PROTO_UNKNOWN;
-            f->alflags |= FLOW_AL_PROTO_DETECT_DONE;
+            f->flags |= FLOW_ALPROTO_DETECT_DONE;
 
-            r = AppLayerParse(f, alproto, f->alflags,
+            r = AppLayerParse(f, alproto, flags,
                            p->payload, p->payload_len);
         } else {
-            f->alflags |= FLOW_AL_PROTO_DETECT_DONE;
+            f->flags |= FLOW_ALPROTO_DETECT_DONE;
             SCLogDebug("ALPROTO_UNKNOWN flow %p", f);
         }
     } else {
@@ -328,7 +515,7 @@ int AppLayerHandleUdp(AlpProtoDetectThreadCtx *dp_ctx, Flow *f, Packet *p)
         /* if we don't have a data object here we are not getting it
          * a start msg should have gotten us one */
         if (alproto != ALPROTO_UNKNOWN) {
-            r = AppLayerParse(f, alproto, f->alflags,
+            r = AppLayerParse(f, alproto, flags,
                         p->payload, p->payload_len);
         } else {
             SCLogDebug(" udp session not start, but no l7 data? Weird");
