@@ -177,7 +177,8 @@ void DetectCsumRegister (void)
  * \param cd  Pointer to the DetectCsumData structure that holds the keyword
  *            value sent as argument
  *
- * \retval 1 if the keyvalue has been parsed successfully, and 0 otherwise
+ * \retval 1 the keyvalue has been parsed successfully
+ * \retval 0 error
  */
 static int DetectCsumParseArg(const char *key, DetectCsumData *cd)
 {
@@ -203,10 +204,10 @@ static int DetectCsumParseArg(const char *key, DetectCsumData *cd)
         return 1;
     }
 
-    SCFree(str);
-    return 0;
 error:
-    return -1;
+    if (str != NULL)
+        SCFree(str);
+    return 0;
 }
 
 /**
@@ -229,7 +230,7 @@ int DetectIPV4CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
 {
     DetectCsumData *cd = (DetectCsumData *)m->ctx;
 
-    if (p->ip4h == NULL)
+    if (p->ip4h == NULL || PKT_IS_PSEUDOPKT(p))
         return 0;
 
     if (p->ip4vars.comp_csum == -1)
@@ -319,14 +320,13 @@ int DetectTCPV4CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
 {
     DetectCsumData *cd = (DetectCsumData *)m->ctx;
 
-    if (p->ip4h == NULL || p->proto != IPPROTO_TCP)
+    if (p->ip4h == NULL || p->proto != IPPROTO_TCP || PKT_IS_PSEUDOPKT(p))
         return 0;
 
     if (p->tcpvars.comp_csum == -1)
         p->tcpvars.comp_csum = TCPCalculateChecksum((uint16_t *)&(p->ip4h->ip_src),
                                                  (uint16_t *)p->tcph,
-                                                 (p->payload_len +
-                                                  p->tcpvars.hlen) );
+                                                 (p->payload_len + TCP_GET_HLEN(p)));
 
     if (p->tcpvars.comp_csum == p->tcph->th_sum && cd->valid == 1)
         return 1;
@@ -411,14 +411,13 @@ int DetectTCPV6CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
 {
     DetectCsumData *cd = (DetectCsumData *)m->ctx;
 
-    if (p->ip6h == NULL || p->proto != IPPROTO_TCP)
+    if (p->ip6h == NULL || p->proto != IPPROTO_TCP || PKT_IS_PSEUDOPKT(p))
         return 0;
 
     if (p->tcpvars.comp_csum == -1)
         p->tcpvars.comp_csum = TCPV6CalculateChecksum((uint16_t *)&(p->ip6h->ip6_src),
                                                    (uint16_t *)p->tcph,
-                                                   (p->payload_len +
-                                                    p->tcpvars.hlen) );
+                                                   (p->payload_len + TCP_GET_HLEN(p)));
 
     if (p->tcpvars.comp_csum == p->tcph->th_sum && cd->valid == 1)
         return 1;
@@ -503,7 +502,7 @@ int DetectUDPV4CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
 {
     DetectCsumData *cd = (DetectCsumData *)m->ctx;
 
-    if (p->ip4h == NULL || p->proto != IPPROTO_UDP)
+    if (p->ip4h == NULL || p->proto != IPPROTO_UDP || PKT_IS_PSEUDOPKT(p))
         return 0;
 
     if (p->udpvars.comp_csum == -1)
@@ -595,7 +594,7 @@ int DetectUDPV6CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
 {
     DetectCsumData *cd = (DetectCsumData *)m->ctx;
 
-    if (p->ip6h == NULL || p->proto != IPPROTO_UDP)
+    if (p->ip6h == NULL || p->proto != IPPROTO_UDP || PKT_IS_PSEUDOPKT(p))
         return 0;
 
     if (p->udpvars.comp_csum == -1)
@@ -687,7 +686,7 @@ int DetectICMPV4CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
 {
     DetectCsumData *cd = (DetectCsumData *)m->ctx;
 
-    if (p->ip4h == NULL || p->proto != IPPROTO_ICMP)
+    if (p->ip4h == NULL || p->proto != IPPROTO_ICMP || PKT_IS_PSEUDOPKT(p))
         return 0;
 
     if (p->icmpv4vars.comp_csum == -1)
@@ -778,7 +777,7 @@ int DetectICMPV6CsumMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
 {
     DetectCsumData *cd = (DetectCsumData *)m->ctx;
 
-    if (p->ip6h == NULL || p->proto != IPPROTO_ICMPV6)
+    if (p->ip6h == NULL || p->proto != IPPROTO_ICMPV6 || PKT_IS_PSEUDOPKT(p))
         return 0;
 
     if (p->icmpv6vars.comp_csum == -1)
@@ -865,10 +864,10 @@ int DetectCsumIPV4ValidArgsTestParse01(void)
     result &= (DetectIPV4CsumSetup(NULL, &s, "\"VALID\"") == 0);
     result &= (DetectIPV4CsumSetup(NULL, &s, "\"iNvaLid\"") == 0);
 
-    while (s.match != NULL) {
-        DetectIPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectIPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -889,10 +888,10 @@ int DetectCsumIPV4InValidArgsTestParse02(void)
     result &= (DetectIPV4CsumSetup(NULL, &s, "VALieD") == -1);
     result &= (DetectIPV4CsumSetup(NULL, &s, "iNvamid") == -1);
 
-    while (s.match != NULL) {
-        DetectIPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectIPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -910,28 +909,28 @@ int DetectCsumIPV4ValidArgsTestParse03(void)
 
     result = (DetectIPV4CsumSetup(NULL, &s, "valid") == 0);
 
-    while (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 1);
         }
-        DetectIPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectIPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
-    s.match = NULL;
+    s.sm_lists[DETECT_SM_LIST_MATCH] = NULL;
 
     result &= (DetectIPV4CsumSetup(NULL, &s, "INVALID") == 0);
 
-    if (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    if (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 0);
         }
-        DetectIPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectIPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -952,10 +951,10 @@ int DetectCsumICMPV4ValidArgsTestParse01(void)
     result &= (DetectICMPV4CsumSetup(NULL, &s, "VALID") == 0);
     result &= (DetectICMPV4CsumSetup(NULL, &s, "iNvaLid") == 0);
 
-    while (s.match != NULL) {
-        DetectICMPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectICMPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -975,10 +974,10 @@ int DetectCsumICMPV4InValidArgsTestParse02(void) {
     result &= (DetectICMPV4CsumSetup(NULL, &s, "VALieD") == -1);
     result &= (DetectICMPV4CsumSetup(NULL, &s, "iNvamid") == -1);
 
-    while (s.match != NULL) {
-        DetectICMPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectICMPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -995,28 +994,28 @@ int DetectCsumICMPV4ValidArgsTestParse03(void) {
 
     result = (DetectICMPV4CsumSetup(NULL, &s, "valid") == 0);
 
-    while (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 1);
         }
-        DetectICMPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectICMPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
-    s.match = NULL;
+    s.sm_lists[DETECT_SM_LIST_MATCH] = NULL;
 
     result &= (DetectICMPV4CsumSetup(NULL, &s, "INVALID") == 0);
 
-    if (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    if (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 0);
         }
-        DetectICMPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectICMPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1037,10 +1036,10 @@ int DetectCsumTCPV4ValidArgsTestParse01(void)
     result &= (DetectTCPV4CsumSetup(NULL, &s, "VALID") == 0);
     result &= (DetectTCPV4CsumSetup(NULL, &s, "iNvaLid") == 0);
 
-    while (s.match != NULL) {
-        DetectTCPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectTCPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1060,10 +1059,10 @@ int DetectCsumTCPV4InValidArgsTestParse02(void) {
     result &= (DetectTCPV4CsumSetup(NULL, &s, "VALieD") == -1);
     result &= (DetectTCPV4CsumSetup(NULL, &s, "iNvamid") == -1);
 
-    while (s.match != NULL) {
-        DetectTCPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectTCPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1080,28 +1079,28 @@ int DetectCsumTCPV4ValidArgsTestParse03(void) {
 
     result = (DetectTCPV4CsumSetup(NULL, &s, "valid") == 0);
 
-    while (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 1);
         }
-        DetectTCPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectTCPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
-    s.match = NULL;
+    s.sm_lists[DETECT_SM_LIST_MATCH] = NULL;
 
     result &= (DetectTCPV4CsumSetup(NULL, &s, "INVALID") == 0);
 
-    if (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    if (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 0);
         }
-        DetectTCPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectTCPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1122,10 +1121,10 @@ int DetectCsumUDPV4ValidArgsTestParse01(void)
     result &= (DetectUDPV4CsumSetup(NULL, &s, "VALID") == 0);
     result &= (DetectUDPV4CsumSetup(NULL, &s, "iNvaLid") == 0);
 
-    while (s.match != NULL) {
-        DetectUDPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectUDPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1145,10 +1144,10 @@ int DetectCsumUDPV4InValidArgsTestParse02(void) {
     result &= (DetectUDPV4CsumSetup(NULL, &s, "VALieD") == -1);
     result &= (DetectUDPV4CsumSetup(NULL, &s, "iNvamid") == -1);
 
-    while (s.match != NULL) {
-        DetectUDPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectUDPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1165,28 +1164,28 @@ int DetectCsumUDPV4ValidArgsTestParse03(void) {
 
     result = (DetectUDPV4CsumSetup(NULL, &s, "valid") == 0);
 
-    while (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 1);
         }
-        DetectUDPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectUDPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
-    s.match = NULL;
+    s.sm_lists[DETECT_SM_LIST_MATCH] = NULL;
 
     result &= (DetectUDPV4CsumSetup(NULL, &s, "INVALID") == 0);
 
-    if (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    if (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 0);
         }
-        DetectUDPV4CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectUDPV4CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1207,10 +1206,10 @@ int DetectCsumTCPV6ValidArgsTestParse01(void)
     result &= (DetectTCPV6CsumSetup(NULL, &s, "VALID") == 0);
     result &= (DetectTCPV6CsumSetup(NULL, &s, "iNvaLid") == 0);
 
-    while (s.match != NULL) {
-        DetectTCPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectTCPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1230,10 +1229,10 @@ int DetectCsumTCPV6InValidArgsTestParse02(void) {
     result &= (DetectTCPV6CsumSetup(NULL, &s, "VALieD") == -1);
     result &= (DetectTCPV6CsumSetup(NULL, &s, "iNvamid") == -1);
 
-    while (s.match != NULL) {
-        DetectTCPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectTCPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1250,28 +1249,28 @@ int DetectCsumTCPV6ValidArgsTestParse03(void) {
 
     result = (DetectTCPV6CsumSetup(NULL, &s, "valid") == 0);
 
-    while (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 1);
         }
-        DetectTCPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectTCPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
-    s.match = NULL;
+    s.sm_lists[DETECT_SM_LIST_MATCH] = NULL;
 
     result &= (DetectTCPV6CsumSetup(NULL, &s, "INVALID") == 0);
 
-    if (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    if (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 0);
         }
-        DetectTCPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectTCPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1292,10 +1291,10 @@ int DetectCsumUDPV6ValidArgsTestParse01(void)
     result &= (DetectUDPV6CsumSetup(NULL, &s, "VALID") == 0);
     result &= (DetectUDPV6CsumSetup(NULL, &s, "iNvaLid") == 0);
 
-    while (s.match != NULL) {
-        DetectUDPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectUDPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1315,10 +1314,10 @@ int DetectCsumUDPV6InValidArgsTestParse02(void) {
     result &= (DetectUDPV6CsumSetup(NULL, &s, "VALieD") == -1);
     result &= (DetectUDPV6CsumSetup(NULL, &s, "iNvamid") == -1);
 
-    while (s.match != NULL) {
-        DetectUDPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectUDPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1335,28 +1334,28 @@ int DetectCsumUDPV6ValidArgsTestParse03(void) {
 
     result = (DetectUDPV6CsumSetup(NULL, &s, "valid") == 0);
 
-    while (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 1);
         }
-        DetectUDPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectUDPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
-    s.match = NULL;
+    s.sm_lists[DETECT_SM_LIST_MATCH] = NULL;
 
     result &= (DetectUDPV6CsumSetup(NULL, &s, "INVALID") == 0);
 
-    if (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    if (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 0);
         }
-        DetectUDPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectUDPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1377,10 +1376,10 @@ int DetectCsumICMPV6ValidArgsTestParse01(void)
     result &= (DetectICMPV6CsumSetup(NULL, &s, "VALID") == 0);
     result &= (DetectICMPV6CsumSetup(NULL, &s, "iNvaLid") == 0);
 
-    while (s.match != NULL) {
-        DetectICMPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectICMPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1400,10 +1399,10 @@ int DetectCsumICMPV6InValidArgsTestParse02(void) {
     result &= (DetectICMPV6CsumSetup(NULL, &s, "VALieD") == -1);
     result &= (DetectICMPV6CsumSetup(NULL, &s, "iNvamid") == -1);
 
-    while (s.match != NULL) {
-        DetectICMPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        DetectICMPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
@@ -1420,28 +1419,28 @@ int DetectCsumICMPV6ValidArgsTestParse03(void) {
 
     result = (DetectICMPV6CsumSetup(NULL, &s, "valid") == 0);
 
-    while (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    while (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 1);
         }
-        DetectICMPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectICMPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
-    s.match = NULL;
+    s.sm_lists[DETECT_SM_LIST_MATCH] = NULL;
 
     result &= (DetectICMPV6CsumSetup(NULL, &s, "INVALID") == 0);
 
-    if (s.match != NULL) {
-        if (s.match->ctx != NULL) {
-            cd = (DetectCsumData *)s.match->ctx;
+    if (s.sm_lists[DETECT_SM_LIST_MATCH] != NULL) {
+        if (s.sm_lists[DETECT_SM_LIST_MATCH]->ctx != NULL) {
+            cd = (DetectCsumData *)s.sm_lists[DETECT_SM_LIST_MATCH]->ctx;
             result &= (cd->valid == 0);
         }
-        DetectICMPV6CsumFree(s.match->ctx);
-        temp = s.match;
-        s.match = s.match->next;
+        DetectICMPV6CsumFree(s.sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+        temp = s.sm_lists[DETECT_SM_LIST_MATCH];
+        s.sm_lists[DETECT_SM_LIST_MATCH] = s.sm_lists[DETECT_SM_LIST_MATCH]->next;
         SCFree(temp);
     }
 
