@@ -1059,6 +1059,13 @@ static int DetectPortParseDo(DetectPort **head, DetectPort **nhead, char *s,
                                                      SC_RULE_VARS_PORT_GROUPS);
                 if (rule_var_port == NULL)
                     goto error;
+                if (strlen(rule_var_port) == 0) {
+                    SCLogError(SC_ERR_INVALID_SIGNATURE, "variable %s resolved "
+                            "to nothing. This is likely a misconfiguration. "
+                            "Note that a negated port needs to be quoted, "
+                            "\"!$HTTP_PORTS\" instead of !$HTTP_PORTS. See issue #295.", s);
+                    goto error;
+                }
                 temp_rule_var_port = rule_var_port;
                 if (negate == 1 || n_set == 1) {
                     temp_rule_var_port = SCMalloc(strlen(rule_var_port) + 3);
@@ -1108,6 +1115,13 @@ static int DetectPortParseDo(DetectPort **head, DetectPort **nhead, char *s,
                                                      SC_RULE_VARS_PORT_GROUPS);
                 if (rule_var_port == NULL)
                     goto error;
+                if (strlen(rule_var_port) == 0) {
+                    SCLogError(SC_ERR_INVALID_SIGNATURE, "variable %s resolved "
+                            "to nothing. This is likely a misconfiguration. "
+                            "Note that a negated port needs to be quoted, "
+                            "\"!$HTTP_PORTS\" instead of !$HTTP_PORTS. See issue #295.", s);
+                    goto error;
+                }
                 temp_rule_var_port = rule_var_port;
                 if ((negate + n_set) % 2) {
                     temp_rule_var_port = SCMalloc(strlen(rule_var_port) + 3);
@@ -1137,8 +1151,20 @@ static int DetectPortParseDo(DetectPort **head, DetectPort **nhead, char *s,
         }
     }
 
+    if (depth > 0) {
+        SCLogError(SC_ERR_INVALID_SIGNATURE, "not every port block was "
+                "properly closed in \"%s\", %d missing closing brackets (]). "
+                "Note: problem might be in a variable.", s, depth);
+        goto error;
+    } else if (depth < 0) {
+        SCLogError(SC_ERR_INVALID_SIGNATURE, "not every port block was "
+                "properly opened in \"%s\", %d missing opening brackets ([). "
+                "Note: problem might be in a variable.", s, depth*-1);
+        goto error;
+    }
+
     return 0;
- error:
+error:
     return -1;
 }
 
@@ -1237,7 +1263,7 @@ int DetectPortParseMergeNotPorts(DetectPort **head, DetectPort **nhead) {
 
             r = DetectPortCmp(ag, ag2);
             if (r == PORT_EQ || r == PORT_EB) { /* XXX more ??? */
-                if (ag2->prev == NULL || ag2 == *head) {
+                if (ag2->prev == NULL) {
                     *head = ag2->next;
                 } else {
                     ag2->prev->next = ag2->next;
@@ -1318,25 +1344,21 @@ error:
  * \retval NULL on error
  */
 DetectPort *PortParse(char *str) {
-    char *portdup = NULL;
+    char *portdup = SCStrdup(str);
     char *port2 = NULL;
     DetectPort *dp = NULL;
-    char *port = NULL;
-
-    if (str == NULL || strlen(str) == 0)
-        return NULL;
-
-    portdup = SCStrdup(str);
-    if (portdup == NULL) {
-        goto error;
-    }
 
     dp = DetectPortInit();
     if (dp == NULL)
         goto error;
 
+    /* XXX better input validation */
+
     /* we dup so we can put a nul-termination in it later */
-    port = portdup;
+    char *port = portdup;
+    if (port == NULL) {
+        goto error;
+    }
 
     /* handle the negation case */
     if (port[0] == '!') {
@@ -1385,8 +1407,7 @@ error:
     if (dp != NULL)
         DetectPortCleanupList(dp);
 
-    if (portdup)
-        SCFree(portdup);
+    if (portdup) SCFree(portdup);
     return NULL;
 }
 
