@@ -36,7 +36,6 @@
 #include "threads.h"
 #include "threadvars.h"
 #include "tm-queuehandlers.h"
-#include "tm-modules.h"
 #include "source-pcap-file.h"
 #include "util-time.h"
 #include "util-debug.h"
@@ -44,11 +43,13 @@
 #include "util-error.h"
 #include "util-privs.h"
 #include "tmqh-packetpool.h"
+#include "tm-threads.h"
+#include "util-optimize.h"
 
 extern uint8_t suricata_ctl_flags;
 extern int max_pending_packets;
 
-static int pcap_max_read_packets = 0;
+//static int pcap_max_read_packets = 0;
 
 typedef struct PcapFileGlobalVars_ {
     pcap_t *pcap_handle;
@@ -59,28 +60,28 @@ typedef struct PcapFileGlobalVars_ {
 } PcapFileGlobalVars;
 
 /** max packets < 65536 */
-#define PCAP_FILE_MAX_PKTS 256
+//#define PCAP_FILE_MAX_PKTS 256
 
 typedef struct PcapFileThreadVars_
 {
     /* counters */
     uint32_t pkts;
     uint64_t bytes;
-    uint32_t errs;
 
     ThreadVars *tv;
+    TmSlot *slot;
 
-    Packet *in_p;
-
-    Packet *array[PCAP_FILE_MAX_PKTS];
-    uint16_t array_idx;
+    /** callback result -- set if one of the thread module failed. */
+    int cb_result;
 
     uint8_t done;
+    uint32_t errs;
 } PcapFileThreadVars;
 
 static PcapFileGlobalVars pcap_g;
 
-TmEcode ReceivePcapFile(ThreadVars *, Packet *, void *, PacketQueue *, PacketQueue *);
+TmEcode ReceivePcapFileLoop(ThreadVars *, void *, void *);
+
 TmEcode ReceivePcapFileThreadInit(ThreadVars *, void *, void **);
 void ReceivePcapFileThreadExitStats(ThreadVars *, void *);
 TmEcode ReceivePcapFileThreadDeinit(ThreadVars *, void *);
@@ -93,11 +94,13 @@ void TmModuleReceivePcapFileRegister (void) {
 
     tmm_modules[TMM_RECEIVEPCAPFILE].name = "ReceivePcapFile";
     tmm_modules[TMM_RECEIVEPCAPFILE].ThreadInit = ReceivePcapFileThreadInit;
-    tmm_modules[TMM_RECEIVEPCAPFILE].Func = ReceivePcapFile;
+    tmm_modules[TMM_RECEIVEPCAPFILE].Func = NULL;
+    tmm_modules[TMM_RECEIVEPCAPFILE].PktAcqLoop = ReceivePcapFileLoop;
     tmm_modules[TMM_RECEIVEPCAPFILE].ThreadExitPrintStats = ReceivePcapFileThreadExitStats;
     tmm_modules[TMM_RECEIVEPCAPFILE].ThreadDeinit = NULL;
     tmm_modules[TMM_RECEIVEPCAPFILE].RegisterTests = NULL;
     tmm_modules[TMM_RECEIVEPCAPFILE].cap_flags = 0;
+    tmm_modules[TMM_RECEIVEPCAPFILE].flags = TM_FLAG_RECEIVE_TM;
 }
 
 void TmModuleDecodePcapFileRegister (void) {
@@ -110,103 +113,88 @@ void TmModuleDecodePcapFileRegister (void) {
     tmm_modules[TMM_DECODEPCAPFILE].cap_flags = 0;
 }
 
-void PcapFileCallback(char *user, struct pcap_pkthdr *h, u_char *pkt) {
+void PcapFileCallbackLoop(char *user, struct pcap_pkthdr *h, u_char *pkt) {
     SCEnter();
 
     PcapFileThreadVars *ptv = (PcapFileThreadVars *)user;
+    Packet *p = PacketGetFromQueueOrAlloc();
 
-    Packet *p = NULL;
-    if (ptv->array_idx == 0) {
-        p = ptv->in_p;
-    } else {
-        p = PacketGetFromQueueOrAlloc();
-    }
-
-    if (p == NULL) {
+    if (unlikely(p == NULL)) {
         SCReturn;
     }
 
     p->ts.tv_sec = h->ts.tv_sec;
     p->ts.tv_usec = h->ts.tv_usec;
     SCLogDebug("p->ts.tv_sec %"PRIuMAX"", (uintmax_t)p->ts.tv_sec);
-    TimeSet(&p->ts);
     p->datalink = pcap_g.datalink;
+    p->pcap_cnt = ++pcap_g.cnt;
 
     ptv->pkts++;
     ptv->bytes += h->caplen;
 
-    p->pktlen = h->caplen;
-    memcpy(p->pkt, pkt, p->pktlen);
-    //printf("PcapFileCallback: p->pktlen: %" PRIu32 " (pkt %02x, p->pkt %02x)\n", p->pktlen, *pkt, *p->pkt);
+    if (unlikely(PacketCopyData(p, pkt, h->caplen))) {
+        TmqhOutputPacketpool(ptv->tv, p);
+        SCReturn;
+    }
 
-    /* store the packet in our array */
-    ptv->array[ptv->array_idx] = p;
-    ptv->array_idx++;
+    if (TmThreadsSlotProcessPkt(ptv->tv, ptv->slot, p) != TM_ECODE_OK) {
+        pcap_breakloop(pcap_g.pcap_handle);
+        ptv->cb_result = TM_ECODE_FAILED;
+    }
 
     SCReturn;
 }
 
 /**
- *  \brief Main PCAP file reading function
+ *  \brief Main PCAP file reading Loop function
  */
-TmEcode ReceivePcapFile(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq, PacketQueue *postpq) {
-    SCEnter();
+TmEcode ReceivePcapFileLoop(ThreadVars *tv, void *data, void *slot) {
     uint16_t packet_q_len = 0;
-
     PcapFileThreadVars *ptv = (PcapFileThreadVars *)data;
+    TmSlot *s = (TmSlot *)slot;
+    ptv->slot = s->slot_next;
+    ptv->cb_result = TM_ECODE_OK;
+    int r;
 
-    if (ptv->done == 1 || suricata_ctl_flags & SURICATA_STOP ||
-            suricata_ctl_flags & SURICATA_KILL) {
-        SCReturnInt(TM_ECODE_FAILED);
-    }
+    SCEnter();
 
-    if (postpq == NULL) {
-        pcap_max_read_packets = 1;
-    }
-
-    ptv->array_idx = 0;
-    ptv->in_p = p;
-
-    /* make sure we have at least one packet in the packet pool, to prevent
-     * us from alloc'ing packets at line rate */
-    do {
-        packet_q_len = PacketPoolSize();
-        if (packet_q_len == 0) {
-            PacketPoolWait();
+    while (1) {
+        if (suricata_ctl_flags & SURICATA_STOP ||
+            suricata_ctl_flags & SURICATA_KILL)
+        {
+            SCReturnInt(TM_ECODE_OK);
         }
-    } while (packet_q_len == 0);
 
-    /* Right now we just support reading packets one at a time. */
-    int r = pcap_dispatch(pcap_g.pcap_handle, (pcap_max_read_packets < packet_q_len) ? pcap_max_read_packets : packet_q_len,
-            (pcap_handler)PcapFileCallback, (u_char *)ptv);
-    if (r < 0) {
-        SCLogError(SC_ERR_PCAP_DISPATCH, "error code %" PRId32 " %s",
-                r, pcap_geterr(pcap_g.pcap_handle));
+        /* make sure we have at least one packet in the packet pool, to prevent
+         * us from alloc'ing packets at line rate */
+        do {
+            packet_q_len = PacketPoolSize();
+            if (unlikely(packet_q_len == 0)) {
+                PacketPoolWait();
+            }
+        } while (packet_q_len == 0);
 
-        /* in the error state we just kill the engine */
-        EngineKill();
-        SCReturnInt(TM_ECODE_FAILED);
-    } else if (r == 0) {
-        SCLogInfo("pcap file end of file reached (pcap err code %" PRId32 ")", r);
+        /* Right now we just support reading packets one at a time. */
+        r = pcap_dispatch(pcap_g.pcap_handle, (int)packet_q_len,
+                (pcap_handler)PcapFileCallbackLoop, (u_char *)ptv);
+        if (unlikely(r == -1)) {
+            SCLogError(SC_ERR_PCAP_DISPATCH, "error code %" PRId32 " %s",
+                    r, pcap_geterr(pcap_g.pcap_handle));
 
-        EngineStop();
-        ptv->done = 1;
+            /* in the error state we just kill the engine */
+            EngineKill();
+            SCReturnInt(TM_ECODE_FAILED);
+        } else if (unlikely(r == 0)) {
+            SCLogInfo("pcap file end of file reached (pcap err code %" PRId32 ")", r);
 
-        /* fall through */
-    }
-
-    uint16_t cnt = 0;
-    for (cnt = 0; cnt < ptv->array_idx; cnt++) {
-        Packet *pp = ptv->array[cnt];
-
-        pcap_g.cnt++;
-
-        if (cnt > 0) {
-            pp->pcap_cnt = pcap_g.cnt;
-            PacketEnqueue(postpq, pp);
-        } else {
-            pp->pcap_cnt = pcap_g.cnt;
+            EngineStop();
+            break;
+        } else if (ptv->cb_result == TM_ECODE_FAILED) {
+            SCLogError(SC_ERR_PCAP_DISPATCH, "Pcap callback PcapFileCallbackLoop failed");
+            EngineKill();
+            SCReturnInt(TM_ECODE_FAILED);
         }
+        SCPerfSyncCountersIfSignalled(tv, 0);
     }
 
     SCReturnInt(TM_ECODE_OK);
@@ -220,11 +208,6 @@ TmEcode ReceivePcapFileThreadInit(ThreadVars *tv, void *initdata, void **data) {
         SCReturnInt(TM_ECODE_FAILED);
     }
 
-    /* use max_pending_packets as pcap read size unless it's bigger than
-     * our size limit */
-    pcap_max_read_packets = (PCAP_FILE_MAX_PKTS < max_pending_packets) ?
-        PCAP_FILE_MAX_PKTS : max_pending_packets;
-
     SCLogInfo("reading pcap file %s", (char *)initdata);
 
     PcapFileThreadVars *ptv = SCMalloc(sizeof(PcapFileThreadVars));
@@ -309,16 +292,21 @@ TmEcode DecodePcapFile(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq, P
     SCPerfCounterIncr(dtv->counter_pkts, tv->sc_perf_pca);
     SCPerfCounterIncr(dtv->counter_pkts_per_sec, tv->sc_perf_pca);
 
-    SCPerfCounterAddUI64(dtv->counter_bytes, tv->sc_perf_pca, p->pktlen);
-    SCPerfCounterAddDouble(dtv->counter_bytes_per_sec, tv->sc_perf_pca, p->pktlen);
+    SCPerfCounterAddUI64(dtv->counter_bytes, tv->sc_perf_pca, GET_PKT_LEN(p));
+#if 0
+    SCPerfCounterAddDouble(dtv->counter_bytes_per_sec, tv->sc_perf_pca, GET_PKT_LEN(p));
     SCPerfCounterAddDouble(dtv->counter_mbit_per_sec, tv->sc_perf_pca,
-                           (p->pktlen * 8)/1000000.0 );
+                           (GET_PKT_LEN(p) * 8)/1000000.0 );
+#endif
+    SCPerfCounterAddUI64(dtv->counter_avg_pkt_size, tv->sc_perf_pca, GET_PKT_LEN(p));
+    SCPerfCounterSetUI64(dtv->counter_max_pkt_size, tv->sc_perf_pca, GET_PKT_LEN(p));
 
-    SCPerfCounterAddUI64(dtv->counter_avg_pkt_size, tv->sc_perf_pca, p->pktlen);
-    SCPerfCounterSetUI64(dtv->counter_max_pkt_size, tv->sc_perf_pca, p->pktlen);
+    /* update the engine time representation based on the timestamp
+     * of the packet. */
+    TimeSet(&p->ts);
 
     /* call the decoder */
-    pcap_g.Decoder(tv, dtv, p, p->pkt, p->pktlen, pq);
+    pcap_g.Decoder(tv, dtv, p, GET_PKT_DATA(p), GET_PKT_LEN(p), pq);
 
     SCReturnInt(TM_ECODE_OK);
 }
