@@ -21,18 +21,18 @@
  * \author Anoop Saldanha <poonaatsoc@gmail.com>
  */
 
-#include <stdio.h>
-#include "threads.h"
-#include <stdint.h>
-
-#include "util-enum.h"
-#include "util-error.h"
-#include "pcre.h"
-#include "util-debug-filters.h"
-
 #ifndef __UTIL_DEBUG_H__
 #define __UTIL_DEBUG_H__
 
+#include <stdio.h>
+#include <stdint.h>
+#include <pcre.h>
+
+#include "threads.h"
+#include "util-enum.h"
+#include "util-error.h"
+#include "util-debug-filters.h"
+
 /**
  * \brief ENV vars that can be used to set the properties for the logging module
  */
@@ -74,7 +74,7 @@ typedef enum {
 #define SC_LOG_DEF_LOG_FORMAT "[%i] %t - (%f:%l) <%d> (%n) -- "
 
 /* The maximum length of the log message */
-#define SC_LOG_MAX_LOG_MSG_LEN 1024
+#define SC_LOG_MAX_LOG_MSG_LEN 2048
 
 /* The maximum length of the log format */
 #define SC_LOG_MAX_LOG_FORMAT_LEN 128
