@@ -45,6 +45,8 @@
                 BUG_ON((p)->udph == NULL);              \
             } else if ((p)->proto == IPPROTO_ICMP) {    \
                 BUG_ON((p)->icmpv4h == NULL);           \
+            } else if ((p)->proto == IPPROTO_SCTP) {    \
+                BUG_ON((p)->sctph == NULL);           \
             } else if ((p)->proto == IPPROTO_ICMPV6) {  \
                 BUG_ON((p)->icmpv6h == NULL);           \
             }                                           \
